Number

Struct Number 

#[repr(C, packed(4))]
pub struct Number { /* private fields */ }
High precision decimal.

Implementations

Source

impl Decimal

pub const ZERO: Decimal

Zero value, i.e. 0.

pub const ONE: Decimal

i.e. 1.

pub const unsafe fn from_parts_unchecked( int_val: u128, scale: i16, negative: bool, ) -> Decimal

Creates a Decimal from parts without boundary checking.

Safety

User have to guarantee that int_val has at most 38 tens digits and scale ranges from [-126, 130].

pub const fn from_parts( int_val: u128, scale: i16, negative: bool, ) -> Result<Decimal, DecimalConvertError>

Creates a Decimal from parts.

int_val has at most 38 tens digits, scale ranges from [-126, 130].

pub const fn into_parts(self) -> (u128, i16, bool)

Consumes the Decimal, returning (int_val, scale, negative).

pub fn precision(&self) -> u8

Returns the precision, i.e. the count of significant digits in this decimal.

pub const fn scale(&self) -> i16

Returns the scale, i.e. the count of decimal digits in the fractional part. A positive scale means a negative power of 10.

pub const fn is_sign_negative(&self) -> bool

Returns true if the sign bit of the decimal is negative.

pub const fn is_sign_positive(&self) -> bool

Returns true if the sign bit of the decimal is positive.

pub const fn is_zero(&self) -> bool

Checks if self is zero.

pub const fn abs(&self) -> Decimal

Computes the absolute value of self.

pub fn encode<W>(&self, writer: W) -> Result<usize, Error>
where W: Write,

Encodes self to writer as binary bytes. Returns total size on success, which is not larger than MAX_BINARY_SIZE.

pub fn compact_encode<W>(&self, writer: W) -> Result<usize, Error>
where W: Write,

Encodes self to writer as binary bytes. Returns total size on success, which is not larger than MAX_BINARY_SIZE.

The only different from Decimal::encode is it will compact encoded bytes when self is zero or small positive integer.

pub fn decode(bytes: &[u8]) -> Decimal

Decodes a Decimal from binary bytes.

pub fn ceil(&self) -> Decimal

Computes the smallest integer that is greater than or equal to self.

pub fn floor(&self) -> Decimal

Computes the largest integer that is equal to or less than self.

pub fn trunc(&self, scale: i16) -> Decimal

Truncate a value to have scale digits after the decimal point. We allow negative scale, implying a truncation before the decimal point.

pub fn round(&self, scale: i16) -> Decimal

Round a value to have scale digits after the decimal point. We allow negative scale, implying rounding before the decimal point.

pub fn round_with_precision(&mut self, precision: u8, scale: i16) -> bool

Do bounds checking and rounding according to precision and scale.

Returns true if overflows.

pub fn normalize_to_scale(&self, scale: i16) -> Decimal

Normalize a Decimal’s scale toward specified scale.

pub fn normalize(&self) -> Decimal

Normalize a Decimal’s scale toward zero.

pub fn checked_add(&self, other: impl AsRef<Decimal>) -> Option<Decimal>

Add two decimals. returning None if overflow occurred.

pub unsafe fn add_with_same_scale_unchecked<const DECIMAL_MODEL: u8>( &self, other: &Decimal, scale: i16, ) -> Decimal

Add two decimals.

Safety

Make sure the decimal is zero or the scale is the same and the result is not overflow.

pub unsafe fn add_with_same_scale_and_negative_unchecked<const DECIMAL_MODEL: u8>( &self, other: &Decimal, scale: i16, negative: bool, ) -> Decimal

Add two decimals.

Safety

Make sure the follow conditions

  1. decimal is zero or the scale is the same.
  2. the result is not overflow.
  3. decimal is zero or the negative is the same.
pub fn checked_sub(&self, other: impl AsRef<Decimal>) -> Option<Decimal>

Subtract one decimal from another, returning None if overflow occurred.

pub unsafe fn sub_with_same_scale_unchecked<const DECIMAL_MODEL: u8>( &self, other: &Decimal, scale: i16, ) -> Decimal

Subtract one decimal from another,

Safety

Make sure two decimal have the same scale or is zero and the result is not overflow.

pub fn checked_mul(&self, other: impl AsRef<Decimal>) -> Option<Decimal>

Calculate the product of two decimals, returning None if overflow occurred.

pub unsafe fn mul_unchecked<const DECIMAL_MODEL: u8>( &self, other: &Decimal, scale: i16, ) -> Decimal

Calculate the product of two decimals,

Safety

Make sure the result scale is scale and the result is not overflow.

pub fn checked_div(&self, other: impl AsRef<Decimal>) -> Option<Decimal>

Checked decimal division. Computes self / other, returning None if other == 0 or the division results in overflow.

pub fn checked_rem(&self, other: impl AsRef<Decimal>) -> Option<Decimal>

Checked decimal remainder. Computes self % other, returning None if rhs == 0 or the division results in overflow.

pub fn sqrt(&self) -> Option<Decimal>

Computes the square root of a decimal, returning None if self is negative or the results in overflow.

pub fn simply_format<W>(&self, w: W) -> Result<(), DecimalFormatError>
where W: Write,

Formats the decimal, including sign and omitting integer zero in fractional.

pub fn format_with_sci<W>( &self, max_width: u16, w: W, ) -> Result<(), DecimalFormatError>
where W: Write,

Formats the decimal, using scientific notation depending on the width.

pub fn format_with_sci_forced<W>( &self, expect_scale: i16, with_zero_before_dot: bool, w: W, ) -> Result<(), DecimalFormatError>
where W: Write,

Formats the decimal, forced using scientific notation depending on the scale.

In particular, the scientific notation is also enforced for 0.
When the decimal is 0 and expect_scale greater than 0, with_zero_before_dot determines whether there is a 0 before the decimal point.

pub fn format_to_hex<W>( &self, is_uppercase: bool, w: W, ) -> Result<(), DecimalFormatError>
where W: Write,

Format decimal as a hexadecimal number.

A maximum of 63 digits hexadecimal positive number are supported.

pub fn format_to_json<W>(&self, w: W) -> Result<(), DecimalFormatError>
where W: Write,

Formats the decimal in the json number format, using scientific notation depending on the width.

pub fn checked_pow(&self, exponent: &Decimal) -> Option<Decimal>

Raise self to the power of exponent, where self and exponent are both decimal, returning None if self == 0 at the same time exponent is negative or self is negative at the same time exponent is a fraction or the result overflowed.

pub fn ln(&self) -> Option<Decimal>

Computes the natural logarithm of self, returning None if self is negative or self == 0.

pub fn exp(&self) -> Option<Decimal>

Computes the nature exponential of self, returning None if the result overflowed.